@@ -1,12 +1,15 @@
 import re
 import pickle
 import unittest
+import warnings
 import importlib
 import importlib_metadata
+import contextlib
 import itertools
 import pyfakefs.fake_filesystem_unittest as ffs
 
 from . import fixtures
+from ._context import suppress
 from importlib_metadata import (
     Distribution,
     EntryPoint,
@@ -20,6 +23,13 @@ from importlib_metadata import (
 )
 
 
+@contextlib.contextmanager
+def suppress_known_deprecation():
+    with warnings.catch_warnings(record=True) as ctx:
+        warnings.simplefilter('default', category=DeprecationWarning)
+        yield ctx
+
+
 class BasicTests(fixtures.DistInfoPkg, unittest.TestCase):
     version_pattern = r'\d+\.\d+(\.\d)?'
 
@@ -44,6 +54,9 @@ class BasicTests(fixtures.DistInfoPkg, unittest.TestCase):
 
         assert "metadata" in str(ctx.exception)
 
+    # expected to fail until ABC is enforced
+    @suppress(AssertionError)
+    @suppress_known_deprecation()
     def test_abc_enforced(self):
         with self.assertRaises(TypeError):
             type('DistributionSubclass', (Distribution,), {})()
